


FUNCTION NombreVersMots( nNum )
   /*
      -------------------------------------------------------------
       Programmer..: Jan-Pierre Martel
       Date........: 04/11/2005
       Notes.......: Converts integer numbers to French word 
                     strings. Limited to numbers from -(10^18) 
                     to 10^19. 
         				Based on Num2Words by Jay Parsons.
       Written for.: Visual dBASE PLUS
       Rev. History: 06/22/1998 - Original Release
                     06/25/1998 - Returns trimmed string 
       Calls.......: convert.Thou2FrenchWords()
       Usage.......: convert.Num2FrenchWords(<nNum>)
       Example.....: ? _app.Convert.Num2FrenchWords( 4321568357 )
       Returns.....: Character string equivalent
       Parameters..: nNum = numeric integer to convert
       Modificarion fevrier 2009
       regle.......: http://fr.wikipedia.org/wiki/Nombres_en_fran%C3%A7ais
      -------------------------------------------------------------
   */

      local nNumleft, nScale, nGroup, cResult

      nNumLeft = int( nNum )    //  prend la partie entiere du nombre
      do case                   //  verifie la taille, si nul, ou negatif

         //case abs( nNumLeft ) >= 10 ^ 18    // bug sur la dernire unit
         //    return("Impossible convertir un nombre plus grand "+;
                     "que 999 999 999 999 999 999")
         case abs( nNumLeft ) >= .3E+16 
              msgbox ("Impossible convertir un nombre plus grand "+;
                     "que 2 999 999 999 999 999", "Limitation")
              return("Impossible de convertir "+str(nNum) +" - limite dpasse")
         case nNumLeft == 0
              return("zro")
         case nNumLeft < 0
              cResult = "moins "
              nNumLeft = - nNumLeft  // transforme nNumLeft en nombre positif
         otherwise
              cResult = ""
      endcase
      do while nNumLeft > 0
         nScale   = int( log10( nNumLeft ) / 3 ) // echelle de grandeur  1 =mille, 2 = million, 3 = milliard...
         nGroup   = int( nNumLeft / 10 ^ ( 3 * nScale ) ) // valeur  (donc <=999) dans l'echelle
         nNumLeft = mod( nNumLeft, 10 ^ ( 3 * nScale ) )  // reste  du nombre
         if nGroup = 1 and nNumleft <> 0    // si une unit non entiere
            cResult += "un "
         else
            //cResult += CLASS::FR_Thou2FrenchWords( nGroup )
            cResult += MoinsDeMilleVersLettres( nGroup )
            if cResult = "un"
               cResult += " "  // ajout pour mille, un million, un milliard...
            endif
         endif
         
         if nNumLeft > 0 and cResult.right(3) == "un "    
            cResult =  cResult.substring( 0, cResult.length )
         endif
         if nScale > 0  // nombre > 999
            cResult += iif(cResult.right(3)=="un ", "", " " );
              + trim( substr( "mille       million     milliard    trillion    quatrillion pentillon   ",;
                nScale * 12 - 11, 12 ) )
              // cherche "un mille" pour supprimer le un
             if right(trim(cResult), 8) = "un mille" and nGroup = 1  // si un mille seul
               cResult = (left(cResult, len(cResult)-8)) + "mille"
             endif
            if nGroup # 1 or nScale < 1
               if right(rtrim(cResult),5) <> "mille"   
                  cResult += "s"            // ajoute un "s" si unit > 1
               else
                  // mille est invariable   
               endif
            endif
            if nNumLeft > 0
               cResult += " "
            endif
         endif
      enddo

   RETURN cResult





FUNCTION MoinsDeMilleVersLettres( nNum )
   /*
      -------------------------------------------------------------
       Programmer..: Jean-Pierre Martel
       Date........: 04/11/2005
       Notes.......: Converts a positive integer less than 1000 to
                     a string of characters.
       Written for.: dBASE PLUS
       Rev. History: Based on Num2Words by Jay Parsons.
                     Modified to display French words
       Calls.......: None
       Usage.......: convert.Thou2FrenchWords(<nNum>)
       Example.....: ? _app.Convert.Thou2FrenchWords( 834 )
       Returns.....: Character string equivalent
       Parameters..: nNum = numeric integer to convert
       Revision....: MA 10 fevrier 2009
      -------------------------------------------------------------
   */
      if nNum > 999          // en cas d'utilisation seul, sans FR_Num2FrencWord
         msgbox("la valeur doit tre inferieure  1000", ;
                "nNum pass = " + str(nNum))
      endif   
      local cUnits, cTens, nN, cResult, nHundreds, cHundreds
      cUnits = "un      deux    " ;
             + "trois   quatre  " ;
             + "cinq    six     " ;
             + "sept    huit    " ;
             + "neuf    dix     " ;
             + "onze    douze   " ;
             + "treize  quatorze" ;
             + "quinze  seize   " ;
             + "dix-septdix-huit" ;
             + "dix-neuf"
      cTens = "dix             vingt           ";
            + "trente          quarante        ";
            + "cinquante       soixante        ";
            + "soixante-dix    quatre-vingt    ";
            + "quatre-vingt-dix"
      nN = mod( int( nNum ), 100 )   // reste de la division par 100
      nHundreds = int(nNum/100)      // nombre de centaines
      if nHundreds > 1        // nombre > 199
         if nNum %100 # 0     // n'est pas un multiple de 100 , pas de "s"
            cHundreds = cUnits.substring((nHundreds-1)*8, nHundreds*8).rightTrim() + " cent "
         else
            cHundreds = cUnits.substring((nHundreds-1)*8, nHundreds*8).rightTrim() + " cents "
         endif
      elseif nHundreds = 1    // nombre entre 100 et 199 inclus
         cHundreds = "cent "
      elseif nHundreds = 0    // nombre inferieur a 100
         cHundreds = ""
      endif
      do case
         case nNum = 0
            return("zro")
         case nNum = 100 and nN = 0
            return("cent")
         case nN = 0
            return( cHundreds.rightTrim() )
         case nN < 20             
            return( cHundreds ;
            + cUnits.substring((nN-1)*8, nN*8).rightTrim() ;
            )
         case nN = 80
            return( cHundreds ;
              + "quatre-vingts";
              )
         case mod(nN,10) = 0
            return( cHundreds ;
              + cTens.substring( (int(nN/10)-1)*16, int(nN/10)*16 ).rightTrim() )
         // case nN = 21 or nN = 31 or nN = 41 or nN = 51 or (nN > 60 and nN < 70)
          case nN = 21 or nN = 31 or nN = 41 or nN = 51 or nN = 61 //(nN > 60 and nN < 70)
            return( cHundreds ;
              + cTens.substring( (int(nN/10)-1)*16, int(nN/10)*16 ).rightTrim() ;
              + " et " ;
              + cUnits.substring( (mod(nN,10)-1)*8, mod(nN,10)*8 ).rightTrim() ;
              )
         case (nN > 69 and nN < 80) and nN <> 71
            return( cHundreds ;
              + cTens.substring( (int(nN/10)-2)*16, (int(nN/10)-1)*16 ).rightTrim() ;
              + "-" ;
              + cUnits.substring( (mod(nN,10)+9)*8, (mod(nN,10)+10)*8 ).rightTrim() ;
              )
         case nN = 71
            return ("soixante et onze")
         case (nN > 90 and nN < 100)
            return( cHundreds ;
              + cTens.substring( (int(nN/10)-2)*16, (int(nN/10)-1)*16 ).rightTrim() ;
              + "-" ; 
              + cUnits.substring( (mod(nN,10)+9)*8, (mod(nN,10)+10)*8 ).rightTrim() ;
              )
         otherwise  // serie 20 30 40 50; 60; 80
            return( cHundreds ;
              + cTens.substring( (int(nN/10)-1)*16, int(nN/10)*16 ).rightTrim() ;
              + "-" ;
              + cUnits.substring( (mod(nN,10)-1)*8, mod(nN,10)*8 ).rightTrim() ;
              )
      endcase
      RETURN

